@@ -1,24 +1,25 @@
-# Standard python packages
 import io
 import json
-import urllib3
-from time import mktime
-from pathlib import Path
-from typing import Union, List
+import mimetypes
+import ssl
 from datetime import datetime, timedelta
+from pathlib import Path
+from time import mktime
+from typing import Union, List
 
-# Django packages
+# noinspection PyPackageRequirements minIO_requirement
+import certifi
+import minio.definitions
+import minio.error
+# noinspection PyPackageRequirements minIO_requirement
+import urllib3
 from django.core.files import File
 from django.core.files.storage import Storage
 from django.core.files.uploadedfile import InMemoryUploadedFile
 from django.utils.deconstruct import deconstructible
 from django.utils.timezone import utc
-# Third-party (MinIO) packages
-import minio.error
-import minio.definitions
 from minio import Minio
 
-# Local Packages
 from .utils import MinioServerStatus, PrivatePublicMixedError, ConfigurationError, get_setting
 
 __all__ = ['MinioBackend', 'get_iso_date', 'iso_date_prefix', ]
@@ -35,6 +36,15 @@ def iso_date_prefix(_, file_name_ext: str) -> str:
 
 @deconstructible
 class MinioBackend(Storage):
+    """
+    :param bucket_name (str): The bucket's name where file(s) will be stored
+    :arg *args: An arbitrary number of arguments. Stored in the self._META_ARGS class field
+    :arg **kwargs: An arbitrary number of key-value arguments.
+        Stored in the self._META_KWARGS class field
+        Through self._META_KWARGS, the "metadata", "sse" and "progress" fields can be set
+        for the underlying put_object() MinIO SDK method
+    """
+
     def __init__(self,
                  bucket_name: str,
                  *args,
@@ -44,6 +54,8 @@ class MinioBackend(Storage):
         self._META_ARGS = args
         self._META_KWARGS = kwargs
 
+        self._REPLACE_EXISTING = kwargs.get('replace_existing', False)
+
         self.__CLIENT: Union[Minio, None] = None
         self.__MINIO_ENDPOINT: str = get_setting("MINIO_ENDPOINT")
         self.__MINIO_ACCESS_KEY: str = get_setting("MINIO_ACCESS_KEY")
@@ -66,7 +78,7 @@ class MinioBackend(Storage):
         django.core.files.storage.Storage
     """
 
-    def _save(self, file_path_name: str, content: InMemoryUploadedFile):
+    def _save(self, file_path_name: str, content: InMemoryUploadedFile) -> str:
         """
         Saves file to Minio by implementing Minio.put_object()
         :param file_path_name (str): Path to file + file name + file extension | ie.: images/2018-12-31/cat.png
@@ -76,6 +88,10 @@ class MinioBackend(Storage):
         # Check if bucket exists, create if not
         self.check_bucket_existence()
 
+        # Check if object with name already exists; delete if so
+        if self._REPLACE_EXISTING and self.stat(file_path_name):
+            self.delete(file_path_name)
+
         # Upload object
         file_path: Path = Path(file_path_name)  # app name + file.suffix
         self.client.put_object(
@@ -83,10 +99,10 @@ class MinioBackend(Storage):
             object_name=file_path.as_posix(),
             data=content,
             length=content.size,
-            content_type=content.content_type,
-            metadata=None,
-            sse=None,
-            progress=None
+            content_type=self._guess_content_type(file_path_name, content),
+            metadata=self._META_KWARGS.get('metadata', None),
+            sse=self._META_KWARGS.get('sse', None),
+            progress=self._META_KWARGS.get('progress', None),
         )
         return file_path.as_posix()
 
@@ -192,6 +208,15 @@ class MinioBackend(Storage):
         obj = self.stat(name)
         return datetime.fromtimestamp(mktime(obj.last_modified))
 
+    @staticmethod
+    def _guess_content_type(file_path_name: str, content: InMemoryUploadedFile):
+        if hasattr(content, 'content_type'):
+            return content.content_type
+        guess = mimetypes.guess_type(file_path_name)[0]
+        if guess is None:
+            return 'application/octet-stream'  # default
+        return guess
+
     """
         MinioBackend
     """
@@ -210,18 +235,20 @@ class MinioBackend(Storage):
             mss.add_message('MINIO_ENDPOINT is not configured in Django settings')
             return mss
 
-        c = urllib3.HTTPSConnectionPool(self.__MINIO_ENDPOINT, cert_reqs='CERT_NONE', assert_hostname=False)
-        try:
-            r = c.request('GET', '/minio/index.html')
-            return MinioServerStatus(r)
-        except urllib3.exceptions.MaxRetryError:
-            mss = MinioServerStatus(None)
-            mss.add_message(f'Could not open connection to {self.__MINIO_ENDPOINT}/minio/index.html ...')
-            return mss
-        except Exception as e:
-            mss = MinioServerStatus(None)
-            mss.add_message(repr(e))
-            return mss
+        with urllib3.PoolManager(cert_reqs=ssl.CERT_REQUIRED, ca_certs=certifi.where()) as http:
+            try:
+                scheme = "https" if self.__MINIO_USE_HTTPS else "http"
+                r = http.request('GET', f'{scheme}://{self.__MINIO_ENDPOINT}/minio/index.html')
+                return MinioServerStatus(r)
+            except urllib3.exceptions.MaxRetryError as e:
+                mss = MinioServerStatus(None)
+                mss.add_message(f'Could not open connection to {self.__MINIO_ENDPOINT}/minio/index.html\n'
+                                f'Reason: {e}')
+                return mss
+            except Exception as e:
+                mss = MinioServerStatus(None)
+                mss.add_message(repr(e))
+                return mss
 
     @property
     def client(self) -> Minio:
